// Copyright (C) 2011-2012 R M Yorston
// Licence: GPLv2+

const Gio = imports.gi.Gio;
const Meta = imports.gi.Meta;

const DND = imports.ui.dnd;
const Main = imports.ui.main;
const WorkspaceThumbnail = imports.ui.workspaceThumbnail;

const OVERRIDE_SCHEMA = 'org.gnome.shell.overrides';
const WORKSPACE_CUT_SIZE = 10;

let settings;
let origHandleDragOver, myHandleDragOver;

function init() {
    settings = new Gio.Settings({ schema: OVERRIDE_SCHEMA });

    // workaround for GNOME Bugzilla #673455
    origHandleDragOver = WorkspaceThumbnail.ThumbnailsBox.prototype.handleDragOver;
    myHandleDragOver = function(source, actor, x, y, time) {
        if (!source.realWindow && !source.shellWorkspaceLaunch && source != Main.xdndHandler)
            return DND.DragMotionResult.CONTINUE;

        let cut = Meta.prefs_get_dynamic_workspaces() ?
                    WORKSPACE_CUT_SIZE : 0;

        let spacing = this.actor.get_theme_node().get_length('spacing');

        this._dropWorkspace = -1;
        let placeholderPos = -1;
        let targetBase;
        if (this._dropPlaceholderPos == 0)
            targetBase = this._dropPlaceholder.y;
        else
            targetBase = this._thumbnails[0].actor.y;
        let targetTop = targetBase - spacing - cut;
        let length = this._thumbnails.length;
        for (let i = 0; i < length; i ++) {
            // Allow the reorder target to have a 10px "cut" into
            // each side of the thumbnail, to make dragging onto the
            // placeholder easier
            let [w, h] = this._thumbnails[i].actor.get_transformed_size();
            let targetBottom = targetBase + cut;
            let nextTargetBase = targetBase + h + spacing;
            let nextTargetTop =  nextTargetBase - spacing - ((i == length - 1) ? 0: cut);

            // Expand the target to include the placeholder, if it exists.
            if (i == this._dropPlaceholderPos)
                targetBottom += this._dropPlaceholder.get_height();

            if (y > targetTop && y <= targetBottom && source != Main.xdndHandler) {
                placeholderPos = i;
                break;
            } else if (y > targetBottom && y <= nextTargetTop) {
                this._dropWorkspace = i;
                break
            }

            targetBase = nextTargetBase;
            targetTop = nextTargetTop;
        }

        if (!Meta.prefs_get_dynamic_workspaces())
            placeholderPos = -1;

        if (this._dropPlaceholderPos != placeholderPos) {
            this._dropPlaceholderPos = placeholderPos;
            this.actor.queue_relayout();
        }

        if (this._dropWorkspace != -1)
            return this._thumbnails[this._dropWorkspace].handleDragOverInternal(source, time);
        else if (this._dropPlaceholderPos != -1)
            return source.realWindow ? DND.DragMotionResult.MOVE_DROP : DND.DragMotionResult.COPY_DROP;
        else
            return DND.DragMotionResult.CONTINUE;
    };
}

function enable() {
    settings.set_boolean('dynamic-workspaces', false);
    WorkspaceThumbnail.ThumbnailsBox.prototype.handleDragOver = myHandleDragOver;
}

function disable() {
    settings.set_boolean('dynamic-workspaces', true);
    WorkspaceThumbnail.ThumbnailsBox.prototype.handleDragOver = origHandleDragOver;
}
